/*
 * gcc -DMODULE -D__KERNEL__ -DLINUX -O -Wall -c module.c -o module.o
 */

#include <linux/config.h>
#include <linux/module.h>

#include <linux/errno.h>
#include <linux/fs.h>
#include <linux/mm.h>
#include <linux/malloc.h>
#include <linux/pci.h>
#include <linux/bios32.h>
#include <linux/major.h>
#include <asm/segment.h>
#include <linux/kernel.h>
#include <linux/signal.h>
#include <linux/pci.h>
#include <linux/bigphysarea.h>

/* name is different in 2.0.x and 2.1.x */
#if LINUX_VERSION_CODE < 0x020100
#define ioremap vremap
#define iounmap vfree
#endif


/* nice and high */
static int major = 42;


/*
 * Our special open code.
 * MOD_INC_USE_COUNT make sure that the driver memory is not freed
 * while the device is in use.
 */
static int mod_open( struct inode* ino, struct file* filep)
{
	return -1;
}

/*
 * Now decrement the use count.
 */
static void mod_close( struct inode* ino, struct file* filep)
{
	MOD_DEC_USE_COUNT;
}

static struct file_operations fops = {
	NULL,			/* lseek */
	NULL,			/* read */
	NULL,			/* write */
	NULL,			/* readdir */
	NULL,			/* select */
	NULL,			/* ioctl */
	NULL,			/*  */
	mod_open,		/* open */
	mod_close,		/* close */
	NULL			/* fsync */
};

#define COUNT 10

void *addr[COUNT];

void test_bigphysarea_alloc()
{
	int i;

	for (i = 0; i < COUNT; i++) {
		addr[i] = bigphysarea_alloc_pages(3, 2, GFP_KERNEL);
		printk("addr[%d] = 0x%08x\n", i, (unsigned)addr[i]);
	}
		
}

void test_bigphysarea_free()
{
	int i;

	for (i = 0; i < COUNT; i++) {
		bigphysarea_free_pages(addr[i]);
	}
}


/*
 * And now the modules code and kernel interface.
 */
int init_module(void)
{
	if (register_chrdev(major, "test-module", &fops)) {
		printk("register_chrdev failed: goodbye world :-(\n");
		return -EIO;
	}

	printk("test-module loaded\n");

	test_bigphysarea_alloc();

	return 0;
}

void cleanup_module(void)
{

	test_bigphysarea_free();

	if (unregister_chrdev(major, "test-module") != 0)
		printk("cleanup_module failed\n");
	else
		printk("test-module removed\n");
}

